% Problem 4.5 LAMINA CHRISTOFFEL EQUATION FOR ISOTROPIC ALUMINUM LAMINA
%{
solve the Christoffel equation for an isotropic aluminum lamina:
for a given wavespeed, find alpha^2, eigenvalues, eigenvectors
check for double alpha^2 roots since we expect identical values for SH and SV
 when double roots are found, expect two orthogonal polarization vectors
repeat for other wavespeeds
a2=alpha^2; a=alpha;
v=wavespeed (avoid vmin<100 because it gives false values due to numerical error)
Note: 
For isotropic materials, alpha^2 has double roots since there is 
no difference between SH&SV waves. But the SH&SV polarization vectors are
orthogonal to each other; this is obtained from the eigenvalue solution of the
Christoffel matrix which has two zero eigenvalues with the corresponding
eigenvectors being orthogonal to each other. 
    The algorithm first checks if double roots occurs within the give tolerance
and exist if this is not true because this would mean that the material 
stiffness matrix does not represent an isotropic material.
    The algorithm proceeds with identifying the double roots which are
treated separately and alocated positions I and II. The third root, which 
represents the P wave, is alocated position III.
    The alpha's and the corresponding polarization vectors are calculated.
Finally, the SH&SV polarization vectors are sorted using the dot product between
them such that consistency and continuity is maintained for all wavespeeds
%}
%% initialization
clc             % clear command window 
clear           % clear workspace
close all       % close all figures
format compact  % densifies command window output lines
%%  initial data: material properties, units, conversion factors, tolerances
deg=180/pi; km_s=1e-3;  % units and convertion factors
[C,rho]=alum_7075_T6;  % stiffness matrix, GPa; density, kg/m3
tol_alpha2=1e-6; % tolerance in alpha^2 when identifying double roots
tol_D=1e-6; % tolerance in D to find the eigenvectors U coresponding to zero D
%% define wavespeed range
vm=100;       % minimum wavespeed, m/s   
                     % (vmin<100 gives false values due to numerical error)
vM=10e3;    % maximum wavespeed, m/s
% Nv=501;     % number of wavespeed points to be considered
Nv=51;     % number of wavespeed points to be considered
if Nv==1; v=vm; else dv=vM/(Nv-1); v=[vm dv:dv:vM]; end
%% or define wavespeed explicitly
% v=[0 1000 2000];
% v=[1900 1907 1920];
% v=1000;
% v=[4000 5000];
% v=[0 5000 8000 12000]; 
% v=[0 5000 8000 12000]; 
% v=[0 1000 3000 4000 6000 8000 12000]; 
% v=[0 1000 2000 4000 6000 8000 10000 12000]; 
% display(v(nv), 'wavespeed, km/s')
Nv=size(v,2); 
%% extract the stiffness coefficients needed in Christoffel equation
C11=C(1,1); C12=C(1,2); C13=C(1,3); C16=C(1,6);
C22=C(2,2); C23=C(2,3); C26=C(2,6);
C33=C(3,3); C36=C(3,6);
C44=C(4,4); C45=C(4,5); C55=C(5,5); C66=C(6,6);
%% loop over all wavespeeds to extract a2, eigenvalues, a, and eigenvectors, U 
% reserve space for eigenvalues
Na=6;Na2=Na/2; a=zeros(Na,Nv); aa2=zeros(Na2,Nv); 
UU=zeros(Na2,Na,Nv); % reserve space for eigenvector
for nv=1:Nv % extract eigenvalues and eigenvectors at each wavespeed
%% write Christoffel eq in expanded form as a cubic eq in a2
rv2=rho*v(nv)^2; rv4=rv2^2; rv6=rv2^3; 
C11_rv2=C11-rv2; C55_rv2=C55-rv2; C66_rv2=C66-rv2;
A6=C33*C44*C55-C33*C45^2;
A4=(C55*C44-C45^2)*C55_rv2+C55*C33*C66_rv2+C44*C33*C11_rv2...
    -2*C16*C45*C33+2*(C45+C36)*(C13+C55)*C45-(C13+C55)^2*C44-(C45+C36)^2*C55;
A2=C33*C11_rv2*C66_rv2+C44*C11_rv2*C55_rv2+C55*C66_rv2*C55_rv2...
    -C11_rv2*(C45+C36)^2-C66_rv2*(C13+C55)^2-2*C55_rv2*C16*C45...
    +2*C16*(C45+C36)*(C13+C55)-C16^2*C33;
A0=(C11_rv2*C66_rv2-C16^2)*C55_rv2;
% display([[A6 A4 A2 A0]'],'A6 A4 A2 A0')
%% scale coefficients of Christoffel eq
sf=max([abs(A6) abs(A4) abs(A2) abs(A0)]); % scale factor for numerical work
% display(sf)
% sf=1; % trial value to see if sf has any influence on accuracy
A6n=A6/sf; A4n=A4/sf; A2n=A2/sf; A0n=A0/sf;
fChr=@(s) (A6*s.^3+A4*s.^2+A2*s+A0)/sf; % the cubic function 
%% solve Christoffel eq
a2=roots([A6n A4n A2n A0n]); f=fChr(a2); % polynomial roots
a2=sort(a2,'descend');   % sort roots in descending order
% display(v(nv),'v, m/s');     % display speed, m/s
% display([a2 f],'a2   f(a2)') % roots and funct values at roots
%% regularize a2 to compensate for numerical error giving false complex values
for na2=1:Na2; 
    if abs(imag(a2(na2)))<=tol_alpha2; a2(na2)=real(a2(na2)); end
end
%% check for double roots
double_flag=0;
for m=1:Na2; 
    for n=m+1:Na2
  if abs(a2(m)-a2(n))<=tol_alpha2; double_flag=1; n_double=n; m_double=m; end
    end ; 
end
%% Calculate eigenvectors
AChr=@(a)...
 [ C11_rv2+C55*a^2       C16+C45*a^2          (C13+C55)*a;
   C16+C45*a^2           C66_rv2+C44*a^2      (C36+C45)*a;
  (C13+C55)*a            (C36+C45)*a           C55_rv2+C33*a^2  ]; % function
%% check for double roots
if double_flag~=1   % check no-double root condition; if true, have to exit
display('select command window and press ctrl+c to stop program execution',...
    'not a double root; not truly isotropic material;')
pause % pause to allow stop program when not a double root
else % this is double-root path   
%% calculate the six values of alpha for double roots
%% find the single root
for n=1:Na2
    if (n~=n_double)&&(n~=m_double); n_single=n; end
end
%% store a2 as aa2
aa2(1,nv)=a2(m_double);
aa2(2,nv)=a2(n_double);
aa2(3,nv)=a2(n_single);
%% calculate the 6 values of alpha and corresponding polarization vectors
%% calculation for the double roots
% calculate the four alphas for the double roots
a(2*m_double-1,nv)=sqrt(a2(m_double));  a(2*m_double,nv)=-sqrt(a2(m_double)); 
a(2*n_double-1,nv)=sqrt(a2(n_double));  a(2*n_double,nv)=-sqrt(a2(n_double)); 
% calculate polarization vectors for for the double roots
icount=0;
    for na=[2*m_double-1 2*m_double]
        icount=icount+1;
        A=AChr(a(na,nv)); An=A./max(max(abs(A)));
        % use eig function 
        % [V,Diag_eigenvalues]=eig(An);
        % use svd function
        [~,Diag_eigenvalues,V]=svd(An);
% find the zero eigenvalues
    indexD1=0; indexD2=0;
    for nD=1:3
        if (abs(Diag_eigenvalues(nD,nD))<=tol_D)&&(indexD1==0); indexD1=nD;
        else
        if (abs(Diag_eigenvalues(nD,nD))<=tol_D)&&(indexD1~=0); indexD2=nD; end         
        end
    end
% pick up in VV the eigenvector corresponding to the zero eigenvalues
    % store in U the normalized VV (max value = 1)  
    if icount==1
    VV=V(:,indexD1); [~,iVVm]=max(abs(VV)); UU(:,1,nv)=VV/VV(iVVm);
    VV=V(:,indexD2); [~,iVVm]=max(abs(VV)); UU(:,3,nv)=VV/VV(iVVm);
    else
    VV=V(:,indexD1); [~,iVVm]=max(abs(VV)); UU(:,2,nv)=VV/VV(iVVm);
    VV=V(:,indexD2); [~,iVVm]=max(abs(VV)); UU(:,4,nv)=VV/VV(iVVm);
    end
    end
%% calculation for the single root
% calculate the two alphas for the single root
a(2*n_single-1,nv)=sqrt(a2(n_single));   a(2*n_single,nv)=-sqrt(a2(n_single));
    % calculate polarization vectors for the single root
icount=0;
    for na=[2*n_single-1 2*n_single]
        icount=icount+1;
        A=AChr(a(na,nv)); An=A./max(max(abs(A)));
        % use eig function 
        % [V,Diag_eigenvalues]=eig(An);
        % use svd function
        [~,Diag_eigenvalues,V]=svd(An);
    % find the zero eigenvalue
            indexD=0;
            for nD=1:3
                if (abs(Diag_eigenvalues(nD,nD))<=tol_D)
                    indexD=nD;
                end
            end
    % pick up in VV the eigenvector corresponding to the zero eigenvalue
    % store in U the normalized VV (max value = 1)  
    if icount==1
       VV=V(:,indexD); [VVm,iVVm]=max(abs(VV)); UU(:,5,nv)=VV/VV(iVVm);
    else
       VV=V(:,indexD); [VVm,iVVm]=max(abs(VV)); UU(:,6,nv)=VV/VV(iVVm);
    end
    end
end
end
%% organize the aa2 values for plotting
%{ 
sort the a2 values for SH&SV at each v according to the resemblance between 
current eigenvector and one of the eigenvectors of previous v;
use only the odd-number eigenvectors (we do NOT sort the a's, we sort the a2's!)
%}
alpha2=zeros(Na2,Nv); U=zeros(Na2,Na,Nv); % reserve space
% the P single root and its two polarization vectors remain in same place
alpha2(3,:)=aa2(3,:); 
U(:,5,:)=UU(:,5,:);
U(:,6,:)=UU(:,6,:);
% first wavespeed is used as reference
% for S double roots, find SH polarization in UU and move it to 3,4 locations
SH=[0 1 0];
[~,~,iUU_SH]=intersect(SH,round(UU(:,:,1))','rows');
if iUU_SH==3 % keep existing order if SH is in 3,4 places
alpha2(:,1)=aa2(:,1); for iS=1:4; U(:,iS,1)=UU(:,iS,1); end   
else % switch places if SH is not in 3,4 places 
alpha2(:,2)=aa2(:,1); alpha2(:,1)=aa2(:,2);
U(:,1,1)=UU(:,3,1); U(:,2,1)=UU(:,4,1);
U(:,3,1)=UU(:,1,1);U(:,4,1)=UU(:,2,1);
end
Na2r=Na2-1; I_select=zeros(Na2r,Nv); % reduce search only to the SH&SV double roots
for nv=2:Nv % loop over all wavespeed pairs
dot_mag=zeros(Na2,1); % reserve space
    for na2=1:Na2r % loop over the SH&SV eigenvectors
        na_odd=2*na2-1; % the odd indices 1,3
        for na1=1:Na2r % compare each 'nv' vector with all 'nv-1' vectors
            na1_odd=2*na1-1; % the odd indices 1,3
            dot_mag(na1)=abs(dot(UU(:,na_odd,nv),U(:,na1_odd,nv-1)));
        end
        [~,I_mag]=max(dot_mag);
%        I_select(na2,nv)=I_select(I_mag,nv-1);
       I_select(na2,nv)=I_mag;
%% pick up alpha2 and Ufrom a2 and UU according to I_select indices
alpha2(I_select(na2,nv),nv)=aa2(na2,nv); 
U(:,2*I_select(na2,nv)-1,nv)=UU(:,2*na2-1,nv);
U(:,2*I_select(na2,nv),nv)=UU(:,2*na2,nv);
        
    end    
end
%% plot and display results
figure(1)
plot(v, alpha2(1,:),v, alpha2(2,:),v, alpha2(3,:));
xlabel('wavespeed, m/s'); ylabel('\alpha^{2}')% note greek and ^ tricks...
legend('\alpha^2_{12}','\alpha^2_{34}','\alpha^2_{56}')
title('Aluminum 7075-T6 isotropic lamina',...
                        'FontSize', 11, 'FontWeight','normal')
grid on
% display([nv,v(nv)],'nv, v(nv)')
% display(U(:,:,nv),'polarization eigenvectors stored in U')
%% store polarization eigenvectors as 


























